@@ -15,8 +15,8 @@
 #include <time.h>
 #include <sys/time.h>
 
-DummyAI::DummyAI(Sea::Player player, Sea* sea)
-: AI(player, sea)
+DummyAI::DummyAI(Sea::Player player, Sea* sea, const BattleShipsConfiguration* config)
+: AI(player, sea, config)
 {
     srand(time(0));
 }
@@ -35,19 +35,3 @@ Coord DummyAI::getMove()
     return desperateMove();
 }
 
-void DummyAI::setShips()
-{
-    // set up computer ships
-    for (int i = 1; i <= 4; i++) {
-        Ship* ship = 0;
-        while (ship == 0) {
-            Coord c(rand() % m_sea->size().x, rand() % m_sea->size().y);
-            Ship::Direction dir = rand() % 2 == 0 ? Ship::LEFT_TO_RIGHT : Ship::TOP_DOWN;
-            if (m_sea->canAddShip(m_player, c, i, dir)) {
-                ship = new Ship(i, dir, c);
-                m_sea->add(m_player, ship);
-            }
-        }
-    }
-}
-
