@@ -14,24 +14,29 @@
 #include "sea.h"
 #include "settings.h"
 
-BattleField::BattleField(Sea* parent, const Coord& size)
+BattleField::BattleField(Sea* parent, const Coord& size, const bool allow_adjacent_ships)
 : QObject(parent)
 , m_size(size)
 , m_board(size)
+, m_secondary_board(size)
+, m_allow_adjacent_ships(allow_adjacent_ships)
 , m_ships(0)
 {
+    setUpSecondaryBoard();
+}
+
+void BattleField::setUpSecondaryBoard()
+{
+    for (int i = 0; i < m_size.x; i++) {
+        for (int j = 0; j < m_size.y ; j++) {
+            m_secondary_board[Coord(i,j)] = false;
+        }
+    }
 }
 
 BattleField::~BattleField()
 {
-	QSet<Ship*> deleted_ships;
-	FOREACH_SQUARE(p, m_board) {
-		Ship* ship = m_board[p].parent();
-		if (ship && !deleted_ships.contains(ship)) {
-			delete ship;
-			deleted_ships.insert(ship);
-		}
-	}
+    clear();
 }
 
 bool BattleField::valid(const Coord& pos) const
@@ -69,6 +74,9 @@ void BattleField::add(Ship* ship)
         p = p + ship->increment();
     }
     m_ships++;
+
+    addSecondaryBoard(ship);
+    addBorderSecondaryBoard(ship);
 }
 
 void BattleField::addBorder(const Coord& pos)
@@ -88,6 +96,40 @@ void BattleField::addBorder(const Coord& pos)
     }
 }
 
+void BattleField::addSecondaryBoard(Ship* ship)
+{
+    Coord p = ship->position();
+    for (unsigned int i = 0; i < ship->size(); i++) {
+        m_secondary_board[p]=true;
+        p += ship->increment();
+    }
+}
+
+void BattleField::addBorderSecondaryBoard(Ship* ship)
+{
+    if ( !m_allow_adjacent_ships )
+    {
+        Coord inc = ship->increment();
+        Coord orth(inc.y, inc.x);
+        Coord p = ship->position() - inc;
+        if ( valid (p) ) {
+            m_secondary_board[p]=true;
+        }
+        for (; p != ship->position() + inc * (ship->size() + 1); p += inc) {
+            if ( valid( p + orth ) ) {
+                m_secondary_board[p + orth]=true;
+            }
+            if ( valid( p - orth ) ) {
+                m_secondary_board[p - orth]=true;
+            }
+        }
+        p -= inc;
+        if ( valid (p) ) {
+            m_secondary_board[p]=true;
+        }
+    }
+}
+
 bool BattleField::canAddShip(const Coord& pos, unsigned int size, Ship::Direction direction) const
 {
     Coord p = pos;
@@ -133,6 +175,56 @@ bool BattleField::canAddShip(const Coord& pos, unsigned int size, Ship::Directio
     return true;
 }
 
+bool BattleField::canAddShipOfSizeInHorizontal(unsigned int size) const
+{
+    unsigned int maxLenAvailable = 0;
+    for (int j=0; j<m_secondary_board.height(); j++) {
+        unsigned int contiguousLen = 0;
+        for (int i=0; i<m_secondary_board.width(); i++) {
+            if ( m_secondary_board[ Coord(i,j) ] ) {
+                contiguousLen = 0;
+            }
+            else {
+                contiguousLen += 1;
+                maxLenAvailable = qMax<int>( contiguousLen, maxLenAvailable );
+                if (maxLenAvailable >= size)
+                {
+                    return true;
+                }
+            }
+        }
+    }
+    return false;
+}
+
+bool BattleField::canAddShipOfSizeInVertical(unsigned int size) const
+{
+    unsigned int maxLenAvailable = 0;
+    for (int i=0; i<m_secondary_board.width(); i++) {
+        unsigned int contiguousLen = 0;
+        for (int j=0; j<m_secondary_board.height(); j++) {
+            if ( m_secondary_board[ Coord(j,i) ] ) {
+                contiguousLen = 0;
+            }
+            else {
+                contiguousLen += 1;
+                maxLenAvailable = qMax<int>( contiguousLen, maxLenAvailable );
+                if (maxLenAvailable >= size)
+                {
+                    return true;
+                }
+            }
+        }
+    }
+    return false;
+}
+
+bool BattleField::canAddShipOfSize(unsigned int size) const
+{
+    return canAddShipOfSizeInHorizontal(size) || canAddShipOfSizeInVertical(size);
+}
+
+
 HitInfo BattleField::hit(const Coord& pos)
 {
     Element& e = get(pos);
@@ -199,4 +291,25 @@ bool BattleField::isNearShip(const Coord& pos) const
     return false;
 }
 
+void BattleField::clear()
+{
+    for (int i = 0; i < m_size.x; i++) {
+        for (int j = 0; j < m_size.y ; j++) {
+            m_secondary_board[Coord(i,j)] = false;
+        }
+    }
+    QSet<Ship*> deleted_ships;
+    FOREACH_SQUARE(p, m_board) {
+        Ship* ship = m_board[p].parent();
+        if (ship && !deleted_ships.contains(ship)) {
+            delete ship;
+            deleted_ships.insert(ship);
+        }
+        m_board[p].setType(Element::WATER);
+        m_board[p].setParent(0);
+    }
+    m_ships=0;
+}
+
+
 #include "battlefield.moc"
