@@ -16,17 +16,18 @@
 #include "seaview.h"
 #include "shot.h"
 #include "audioplayer.h"
+#include "ships.h"
 
-Controller::Controller(QObject* parent, AudioPlayer* player, const bool allow_adjacent_ships)
+Controller::Controller(QObject* parent, AudioPlayer* audioPlayer, const BattleShipsConfiguration& battleShipsConfiguration)
 : QObject(parent)
 , m_shot(0)
 , m_ready(0)
-, m_player(player)
+, m_player(audioPlayer)
 , m_has_ai(false)
-, m_allow_adjacent_ships(allow_adjacent_ships)
+, mBattleShipsConfiguration(battleShipsConfiguration)
 {
     m_ui = 0;
-    m_sea = new Sea(this, Coord(10, 10), allow_adjacent_ships);
+    m_sea = new Sea(this, battleShipsConfiguration);
 }
 
 PlayerEntity* Controller::createPlayer(Sea::Player player, SeaView* view,
@@ -68,17 +69,21 @@ NetworkEntity* Controller::createRemotePlayer(Sea::Player player, SeaView* view,
 void Controller::setupEntity(Entity* entity)
 {
     entity->setParent(this);
-    
+
     connect(entity, SIGNAL(shoot(int,Coord)),
             this, SLOT(shoot(int,Coord)), Qt::QueuedConnection);
     connect(entity, SIGNAL(ready(int)),
-            this, SLOT(ready(int)), Qt::QueuedConnection);
+            this, SLOT(ready(int)));
+    connect(entity, SIGNAL(shipsPlaced(int)),
+            this, SLOT(shipsPlaced(int)));
     connect(entity, SIGNAL(chat(QString)),
             this, SLOT(receivedChat(QString)));
     connect(entity, SIGNAL(nick(int,QString)),
             this, SLOT(nick(int,QString)));
     connect(entity, SIGNAL(compatibility(int)),
             this, SIGNAL(compatibility(int)));
+    connect(entity, SIGNAL(gameOptionsInterchanged(bool)),
+            this, SLOT(placing(bool)));
 
     foreach (Entity* e, m_entities) {
         connect(e, SIGNAL(compatibility(int)),
@@ -90,11 +95,21 @@ void Controller::setupEntity(Entity* entity)
                 entity, SLOT(notifyAbort()));
         connect(entity, SIGNAL(abortGame()),
                 e, SLOT(notifyAbort()));
+        connect(e, SIGNAL(restartPlacingShips(Sea::Player)),
+                this, SIGNAL(restartPlacingShips(Sea::Player)));
+        connect(e, SIGNAL(restartPlacingShips(Sea::Player)),
+                this, SLOT(notifyRestartPlacingShips(Sea::Player)));
     }
 
     m_entities.append(entity);
 }
 
+void Controller::setBattleShipsConfiguration(const BattleShipsConfiguration& battleConfiguration)
+{
+    mBattleShipsConfiguration = battleConfiguration;
+}
+
+
 bool Controller::allPlayers() const
 {
     unsigned char bitmap = 0;
@@ -113,14 +128,14 @@ bool Controller::start(SeaView* view, bool ask)
     if (!allPlayers()) {
         return false;
     }
-    
+
     if (!m_ui) {
         m_ui = new UIEntity(Sea::NO_PLAYER, m_sea, view);
         setupEntity(m_ui);
     }
-    
+
     foreach (Entity* entity, m_entities) {
-        entity->start(ask);
+        entity->notifyGameOptions(ask);
     }
     
     foreach (Entity* source, m_entities) {
@@ -131,9 +146,40 @@ bool Controller::start(SeaView* view, bool ask)
             }
         }
     }
+
     return true;
 }
 
+void Controller::restart(bool ask)
+{
+    m_ready = 0;
+    if (ask)
+    {
+        foreach (Entity* entity, m_entities) {
+            entity->notifyRestart(entity->player());
+        }
+    }
+
+    m_sea->clear(Sea::PLAYER_A);
+    m_sea->clear(Sea::PLAYER_B);
+
+    foreach (Entity* entity, m_entities) {
+        m_sea->clear(entity->player());
+            emit startPlacingShips(Sea::PLAYER_A);
+            entity->startPlacing(false);
+    }
+}
+
+
+// It is sure the entities has interchanged the GameOptions (if any)
+// when the opposite nick is received
+void Controller::placing(bool ask)
+{
+    foreach (Entity* entity, m_entities) {
+        entity->startPlacing(ask);
+    }
+}
+
 void Controller::shoot(int player, const Coord& c)
 {
     Entity* entity = findEntity(Sea::opponent(Sea::Player(player)));
@@ -147,7 +193,7 @@ void Controller::shoot(int player, const Coord& c)
         // shot in progress
         return;
     }
-    
+
     if (m_sea->status() == Sea::PLAYING) {
         entity->hit(m_shot = new Shot(this, Sea::Player(player), c)); // kind of CPS
     }
@@ -158,12 +204,12 @@ void Controller::finalizeShot(Sea::Player player, const Coord& c, const HitInfo&
     if (info.type != HitInfo::INVALID) {
         // notify entities
         notify(player, c, info);
-        
+
         // play sounds
         if (m_player) {
             m_player->play(player, info);
         }
-            
+
         if (m_sea->status() == Sea::A_WINS) {
             finalizeGame(Sea::PLAYER_A);
         }
@@ -192,17 +238,29 @@ void Controller::notify(Sea::Player player, const Coord& c, const HitInfo& info)
     }
 }
 
+void Controller::shipsPlaced(int player)
+{
+    m_ready++;
+    if (m_ready >= 2 )
+    {
+        foreach (Entity* entity, m_entities) {
+            entity->start(false);
+        }
+    }
+}
+
+
 void Controller::ready(int player)
 {
     m_ready++;
     foreach (Entity* entity, m_entities) {
         entity->notifyReady(Sea::Player(player));
     }
-    
-    // when two entities are ready, start
-    // all engines
-    if (m_ready >= 2) {
+    // when two entities are ready (ships placed and ready)
+    // start all engines
+    if (m_ready >= 4) {
         m_sea->startPlaying();
+
         foreach (Entity* entity, m_entities) {
             entity->startPlaying();
         }
@@ -224,7 +282,16 @@ void Controller::finalizeGame(Sea::Player winner)
         entity->notifyGameOver(winner);
     }
     emit gameOver(winner);
- }
+}
+
+void Controller::notifyRestartPlacingShips(Sea::Player player)
+{
+    foreach (Entity* entity, m_entities) {
+        if (entity->player() == player) {
+            entity->notifyRestartPlacing(player);
+        }
+    }
+}
 
 Entity* Controller::findEntity(Sea::Player player) const
 {
