@@ -30,7 +30,9 @@ public:
     virtual void notifyGameOver(Sea::Player winner);
     virtual void notifyChat(const Entity*, const QString&) { }
     virtual void notifyNick(Sea::Player, const QString&) { }
+    virtual void notifyGameOptions(bool ask) { emit gameOptionsInterchanged(); }
     virtual void start(bool);
+    virtual void startPlacing(bool) { };
     virtual void hit(Shot*);
     
     virtual void registerHit(Sea::Player, const Coord&) { }
@@ -39,6 +41,7 @@ public:
     virtual KIcon icon() const;
 public slots:
     virtual void notifyAbort() { }
+    virtual void notifyRestartPlacing(Sea::Player player) { };
 };
 
 #endif // UIENTITY_H
